@@ -1,5 +1,5 @@
 # $File: //depot/cpanplus/dist/lib/CPANPLUS/Error.pm $
-# $Revision: #3 $ $Change: 1963 $ $DateTime: 2002/11/04 16:32:10 $
+# $Revision: #7 $ $Change: 11204 $ $DateTime: 2004/09/20 20:15:05 $
 
 ###############################################
 ###              CPANPLUS/Error.pm          ###
@@ -47,6 +47,8 @@ sub new {
                         ? $args{error_level}
                         : 1,
         COUNTER => 0,
+        ECALLBACK => undef,
+        ICALLBACK => undef,
     };
     return bless $self, $class;
 }
@@ -83,6 +85,8 @@ sub trap {
         carp($self->{ERROR}) if ($self->{ELEVEL});
     }
 
+    $self->{ECALLBACK}->($self->{ERROR}) if $self->{ECALLBACK};
+
     ### Return that the error was handled ###
     return 1;
 
@@ -146,6 +150,8 @@ sub inform {
 
     $self->{MSG} = $msg;
 
+    $self->{ICALLBACK}->($self->{MSG}) if $self->{ICALLBACK};
+
     ### indicate the error was handled
     return 1;
 }
