@@ -1,5 +1,5 @@
 # $File: //depot/cpanplus/dist/lib/CPANPLUS/Internals/Report.pm $
-# $Revision: #12 $ $Change: 7687 $ $DateTime: 2003/08/24 03:37:55 $
+# $Revision: #16 $ $Change: 11204 $ $DateTime: 2004/09/20 20:15:05 $
 
 ####################################################
 ###          CPANPLUS/Internals/Report.pm        ###
@@ -97,8 +97,8 @@ sub _send_report {
     return unless length $dist
               and $self->_can_use( modules => { 'Test::Reporter' => '1.13' } )
               and $self->{_shell}
-              and $self->{_shell}->_ask_report(dist => $dist, grade => $grade)
-              and $self->_not_already_sent($dist);
+              and $self->_not_already_sent($dist)
+              and $self->{_shell}->_ask_report(dist => $dist, grade => $grade);
 
     my ($already_sent, $nb_send, $max_send) = (0, 0, 2);
     my @inform;
@@ -147,6 +147,16 @@ Additional comments:
             } sort keys %missing);
 
             $text .= << ".";
+******************************** NOTE ********************************
+The comments below are created mechanically, possibly without manual
+checking by the sender.  Also, because many people perform automatic
+tests on CPAN, chances are that you will receive identical messages
+about the same problem.
+
+If you believe that the message is mistaken, please reply to the first
+one with correction and/or additional information, and do not take
+it personally.  We appreciate your patience. :)
+**********************************************************************
 
 Hello, $author! Thanks for uploading your works to CPAN.
 
@@ -167,22 +177,11 @@ $prereq
 
 If you are interested in making a more flexible Makefile.PL that can
 probe for missing dependencies and install them, ExtUtils::AutoInstall
-at <http://search.cpan.org/search?dist=ExtUtils-AutoInstall> may be
+at <http://search.cpan.org/dist/ExtUtils-AutoInstall/> may be
 worth a look.
 
 Thanks! :-)
 
-******************************** NOTE ********************************
-The comments above are created mechanically, possibly without manual
-checking by the sender.  Also, because many people perform automatic
-tests on CPAN, chances are that you will receive identical messages
-about the same problem.
-
-If you believe that the message is mistaken, please reply to the first
-one with correction and/or additional information, and do not take
-it personally.  We appreciate your patience. :)
-**********************************************************************
-
 .
         }
 
@@ -226,7 +225,7 @@ __END__
 
 would be appreciated.  If you are interested in making a more robust
 test suite, please see the Test::Simple, Test::More and Test::Tutorial
-manpages at <http://search.cpan.org/search?dist=Test-Simple>.
+manpages at <http://search.cpan.org/dist/Test-Simple/>.
 
 Thanks! :-)
 
